// BigRIPS/ZeroDegree calibration class
// 

#ifndef _TARTCALIBPID_H_
#define _TARTCALIBPID_H_

#include <map>
#include <TNamed.h>

class TArtRawEventObject;
class TArtBigRIPSParameters;
class TArtMapper;

class TArtPPAC;
class TArtPPACArray;
class TArtPlastic;
class TArtPlasticArray;
class TArtIC;
class TArtICArray;

class TArtCalibPID  {

 public:

  TArtCalibPID();
  TArtCalibPID(TArtBigRIPSParameters * s);
  virtual ~TArtCalibPID();

  TArtPPACArray    * GetPPAC(){return fPPAC;}
  TArtPPAC         * GetPPAC(int i);
  TArtPlasticArray * GetPlastic(){return fPlastic;}
  TArtPlastic      * GetPlastic(int i);
  TArtICArray      * GetIC(){return fIC;}
  TArtIC           * GetIC(int i);

  Int_t GetNumPPAC();
  Int_t GetNumPlastic();
  Int_t GetNumIC();

  void      LoadRawData(TArtRawEventObject *);
  void      ClearData();
  void      ReconstructData();
  void      ReconstructData(TArtRawEventObject *);

  bool IsReconstructed(){ return fReconstructed; }

 private:

  void Initialize();
  bool      fReconstructed;

  // Subsystems objects used for PID
  TArtPPACArray    * fPPAC;
  TArtICArray      * fIC;
  TArtPlasticArray * fPlastic;

  TArtBigRIPSParameters * setup;
  TArtMapper *mapper;

  ClassDef(TArtCalibPID,1);

 };
#endif
