#ifndef _TARTNEBULAPARA_H_
#define _TARTNEBULAPARA_H_

#include <Riostream.h>

#include "TArtMap.h"
#include "segidlist.h"

class TArtNEBULAPara : public TNamed
{
 public:
  TArtNEBULAPara(Int_t i, TString n, Int_t f, Int_t l, Int_t sl,
		 Double_t tc, Double_t to, Double_t qc, Double_t qp, Int_t ig)
    : id(i), detname(n), fpl(f), layer(l), sublayer(sl),
    tcal(tc), toff(to), qcal(qc), qped(qp), ignore(ig){;}
  ~TArtNEBULAPara(){;}

  void SetMap(Int_t tgeo, Int_t tch, Int_t qgeo, Int_t qch)
  {
    map_adc.SetMap(fpl,42,qgeo,qch); 
    //    map_adc.SetMap(fpl,NEBULAA,qgeo,qch); 
    //    cout << "ADC is mapped to " << map_adc.GetMapInfo() << " " << map_adc.GetName() << endl;
    map_tdc.SetMap(fpl,41,tgeo,tch); 
    //    map_tdc.SetMap(fpl,NEBULAT,tgeo,tch); 
    //    cout << "TDC is mapped to " << map_tdc.GetMapInfo() << " " << map_tdc.GetName() << endl;
  }    

  Int_t GetID() const { return id; }
  const TString& GetDetectorName() const { return detname; }
  Int_t GetFpl() const { return fpl; }
  Int_t GetLayer() const { return layer; }
  Int_t GetSubLayer() const { return sublayer; }
  Double_t GetTCal() const { return tcal; } 
  Double_t GetTOffset() const { return toff; } 
  Double_t GetQCal() const { return qcal; } 
  Double_t GetQPed() const { return qped; } 
  //  Double_t GetPos(int i) const { return pos[i]; } 
  Int_t GetIgnore() const { return ignore; }

  TArtRIDFMap * GetTDCMap() { return &map_tdc;  }
  TArtRIDFMap * GetADCMap() { return &map_adc;  }

  friend ostream & operator << (ostream& out, const TArtNEBULAPara& p) {
    out << "ID: " << p.id << endl;
    out << "DetectorName: " << p.detname << endl;
    out << "Focal Plane: " << p.fpl << endl;
    out << "Layer: " << p.layer << endl;
    out << "SubLayer: " << p.sublayer << endl;
    out << "Time Calib: " << p.tcal << endl;
    out << "Time Offset: " << p.toff << endl;
    out << "QDC Calib: " << p.qcal << endl;
    out << "QDC pedestal: " << p.tcal << endl;
    //    out << "Position: " <<p.pos[0]<< " " <<p.pos[1]<< " " <<p.pos[2]<< endl;
    out << "Ignore: " << p.ignore << endl;
    out << "Map of ADC: "  << p.map_adc << endl;
    out << "Map of TDC: "  << p.map_tdc << endl;
    out << endl;
    return out;
  }

 private:
  Int_t     id;
  TString   detname;
  Int_t     fpl;
  Int_t     layer;
  Int_t     sublayer;
  Double_t  tcal;
  Double_t  toff;
  Double_t  qcal;
  Double_t  qped;
  //  Double_t  pos[3]; // position in mm
  Int_t     ignore;

  TArtRIDFMap map_tdc;
  TArtRIDFMap map_adc;

  ClassDef(TArtNEBULAPara,1);

};

#endif
