
#include "TArtParameterNEBULA.hh"

#include "TArtCore.h"
#include "TArtMap.h"

#include <TDOMParser.h>
#include <TXMLNode.h>

#include <stdlib.h>
#include <iostream>
#include <fstream>

//__________________________________________________________
TArtParameterNEBULA::TArtParameterNEBULAInner::TArtParameterNEBULAInner()
{;}

//__________________________________________________________
TArtParameterNEBULA::TArtParameterNEBULAInner::TArtParameterNEBULAInner
(Int_t id, TString detname, Int_t fpl,
 Int_t layer, Int_t sublayer,
 Double_t tcal, Double_t toff,
 Double_t qcal, Double_t qped,
 Int_t ignore)
  : fID(id), fDetName(detname), fFPl(fpl), fLayer(layer), fSubLayer(sublayer),
    fTCal(tcal), fTOff(toff), fQCal(qcal), fQPed(qped), fIgnore(ignore)
{;}

//__________________________________________________________
TArtParameterNEBULA::TArtParameterNEBULAInner::~TArtParameterNEBULAInner()
{;}

//__________________________________________________________
void TArtParameterNEBULA::TArtParameterNEBULAInner::SetMap
(Int_t tgeo, Int_t tch, Int_t qgeo, Int_t qch)
{
  fMapADC.SetMap(fFPl,42,qgeo,qch); 
  std::cout << "ADC is mapped to " << fMapADC.GetMapInfo() << " "
	    << fMapADC.GetName() << std::endl;
  fMapTDC.SetMap(fFPl,41,tgeo,tch); 
  std::cout << "TDC is mapped to " << fMapTDC.GetMapInfo() << " "
	    << fMapTDC.GetName() << std::endl;
}    

//__________________________________________________________
TArtParameterNEBULA::TArtParameterNEBULA(char *xmlfile)
  : TArtParameter()
{
  TArtParameter::LoadParameter(xmlfile);
}

//__________________________________________________________
TArtParameterNEBULA::~TArtParameterNEBULA() 
{;}

//__________________________________________________________
void TArtParameterNEBULA::ParseParaList(TXMLNode *node) {
  for (; node; node = node->GetNextNode()) {
    if (node->GetNodeType() == TXMLNode::kXMLElementNode) { // Element Node
      if (strcmp(node->GetNodeName(), "NEBULA") == 0){
	TArtParameterNEBULAInner para = ParseNEBULAPara(node->GetChildren());
	fPMap.insert(std::pair<TArtRIDFMap, TArtParameterNEBULAInner>
		    (para.fMapADC, para));
	fPMap.insert(std::pair<TArtRIDFMap, TArtParameterNEBULAInner>
		    (para.fMapTDC, para));
      }
    }
  }
  return;
}

//__________________________________________________________
TArtParameterNEBULA::TArtParameterNEBULAInner TArtParameterNEBULA::ParseNEBULAPara(TXMLNode *node)
{
  Int_t     id = -1;
  TString   name;
  Int_t     fpl = -1;
  Int_t     layer = -1;
  Int_t     sublayer = -1;
  Double_t  tcal = 0;
  Double_t  toff = 0;
  Double_t  qcal = 0;
  Double_t  qped = 0;
  Int_t     ignore = 0;
  Int_t     tdc_geo, tdc_ch, adc_geo, adc_ch;

  for ( ; node; node = node->GetNextNode()) {
    if (node->GetNodeType() == TXMLNode::kXMLElementNode) { // Element Node
      if (strcmp(node->GetNodeName(), "Id") == 0) 
	id = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "NAME") == 0) 
	name = node->GetText();
      if (strcmp(node->GetNodeName(), "FPL") == 0) 
	fpl = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "layer") == 0) 
	layer = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "sublayer") == 0) 
	sublayer = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "tcal") == 0)
	tcal = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "toff") == 0)
	toff = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "qcal") == 0)
	qcal = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "qped") == 0)
	qped = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "ignore") == 0)
	ignore = atoi(node->GetText());

      if (strcmp(node->GetNodeName(), "tdc_geo") == 0) 
	tdc_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "tdc_ch") == 0) 
	tdc_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "adc_geo") == 0) 
	adc_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "adc_ch") == 0) 
	adc_ch = atoi(node->GetText());
    }
  }
  
  TArtCore::Info(__FILE__,"Reading database for %s NEBULA",name.Data());
  TArtParameterNEBULAInner para(id, name, fpl, layer, sublayer, tcal, toff, qcal, qped, ignore);
  para.SetMap(tdc_geo, tdc_ch, adc_geo, adc_ch);

  return para;

}

//__________________________________________________________
const TArtParameterNEBULA::TArtParameterNEBULAInner* TArtParameterNEBULA::Find(const TArtRIDFMap& map)
{
  std::map<TArtRIDFMap, TArtParameterNEBULAInner>::iterator it = fPMap.find(map);
  if(it != fPMap.end()){
    return static_cast<const TArtParameterNEBULAInner*>(&(it->second));
  }else{
    return 0;
  }
}
