
#include "TArtCanvasManager.hh"

#include "TROOT.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TVirtualPad.h"
#include "TList.h"

#include <iostream>

void TArtCanvasManager::SetLogx(Int_t value)
{
  gPad->SetLogx(value);
}

void TArtCanvasManager::SetLogy(Int_t value)
{
  gPad->SetLogy(value);
}

void TArtCanvasManager::SetLogz(Int_t value)
{
  gPad->SetLogz(value);
}

TCanvas* TArtCanvasManager::GetCanvas()
{
//  TList* list = static_cast<TList*>(gROOT->GetListOfCanvases());
//  TObject* obj = list->Last();
//  if(obj && obj->InheritsFrom(TCanvas::Class())){
//    return static_cast<TCanvas*>(obj);
//  }else{
//    return 0;
//  }
  if(!gPad) return 0;
  return gPad->GetCanvas();
}

TCanvas* TArtCanvasManager::GetCanvas(Int_t idx)
{
  TList* list = static_cast<TList*>(gROOT->GetListOfCanvases());

  TObject* obj = list->At(idx);
  if(obj && obj->InheritsFrom(TCanvas::Class())){
    return static_cast<TCanvas*>(obj);
  }else{
    return 0;
  }
}

void TArtCanvasManager::SetWindowSize(UInt_t ww, UInt_t wh)
{
  if(ww < 100 || wh < 100) return;

  TCanvas* canvas = GetCanvas();
  if(!canvas) return;

  canvas->SetWindowSize(ww, wh);
}

void TArtCanvasManager::Divide(Int_t nx, Int_t ny, Float_t xmargin, Float_t ymargin, Int_t color)
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return;

  if(nx < 1 || ny < 1){
    std::cout << "nx and ny must be positive number" << std::endl;
    return;
  }

  canvas->Clear();
  canvas->Divide(nx, ny, xmargin, ymargin, color);
  canvas->cd(1);
}

TVirtualPad* TArtCanvasManager::cd(Int_t subpadnumber)
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return 0;  
  TList* list = canvas->GetListOfPrimitives();
  if(!list) return 0;
  return canvas->cd((subpadnumber-1)%list->GetEntries()+1);
}

TVirtualPad* TArtCanvasManager::cdnext()
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return 0;
  TList* list = canvas->GetListOfPrimitives();
  if(!list) return 0;
  TObjLink* objlink = list->FirstLink();
  while(objlink){
    TObject* obj = objlink->GetObject();
    if(obj && obj->InheritsFrom(TPad::Class())){
      TPad* pad = static_cast<TPad*>(obj);
      if(pad->IsEqual(gPad)){
	objlink = objlink->Next();      
	if(!objlink) objlink = list->FirstLink();
	TPad* pad = static_cast<TPad*>(objlink->GetObject());
	return pad->cd();
      }
    }
    objlink = objlink->Next();
  }  

  return 0;
}

TVirtualPad* TArtCanvasManager::cdprev()
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return 0;
  TList* list = canvas->GetListOfPrimitives();
  if(!list) return 0;
  TObjLink* objlink = list->LastLink();
  while(objlink){
    TObject* obj = objlink->GetObject();
    if(obj && obj->InheritsFrom(TPad::Class())){
      TPad* pad = static_cast<TPad*>(obj);
      if(pad->IsEqual(gPad)){
	objlink = objlink->Prev();      
	if(!objlink) objlink = list->LastLink();
	TPad* pad = static_cast<TPad*>(objlink->GetObject());
	return pad->cd();
      }
    }
    objlink = objlink->Prev();
  }  

  return 0;
}
