
#include "TArtExec.hh"

#include "TH1.h"
#include "TH2.h"
#include "TPad.h"
#include "TCanvas.h"
#include "TROOT.h"
#include "Buttons.h"
#include "TSystem.h"

#include <iostream>
#include <iomanip>

Double_t TArtExec::GXValue()
{
  FGXValue gxvalue;
  EventLoop(gxvalue);
  return gxvalue.GetX();
}

Double_t TArtExec::GYValue()
{
  FGYValue gyvalue;
  EventLoop(gyvalue);
  return gyvalue.GetY();
}

void TArtExec::GXYValue(Double_t& x, Double_t& y)
{
  FGXYValue gxyvalue;
  EventLoop(gxyvalue);
  x = gxyvalue.GetX();
  y = gxyvalue.GetY();
}

//explicit instantiation
template void TArtExec::EventLoop<TArtExec::FGXValue>(TArtExec::FGXValue&);
template void TArtExec::EventLoop<TArtExec::FGYValue>(TArtExec::FGYValue&);
template void TArtExec::EventLoop<TArtExec::FGXYValue>(TArtExec::FGXYValue&);

template<class T>
void TArtExec::EventLoop(T& fpt)
{
  if(!gPad) return;
  TCanvas* canvas = gPad->GetCanvas();
  if(!canvas) return;
  canvas->HandleInput((EEventType)-1,0,0);

  fpt(kInitial);
  while (!gSystem->ProcessEvents()){
    EEventType event = static_cast<EEventType>(canvas->GetEvent());
    if(event == kButton1Down || event == kButton1Double){
      fpt(kEnd);
      canvas->HandleInput((EEventType)-1,0,0);
      break;
    }
    fpt(kRunning);
    gSystem->Sleep(10);
  }
}

void TArtExec::FGXValue::operator()(EExecState execstate)
{
  if(!gPad) return;
  TObject *select = gPad->GetSelected();
  if(!select) return;
  if(!select->InheritsFrom(TH1::Class())){ return;}

  TCanvas* canvas = gPad->GetCanvas();
  if(!canvas) return;
  if(execstate != kEnd) canvas->FeedbackMode(kTRUE);
  else canvas->FeedbackMode(kFALSE);

  static int px;
  int pxold = px;
  px = gPad->GetEventX();
  std::cout << "\r" << (fX = gPad->AbsPixeltoX(px)) << std::flush;
  if(execstate == kEnd) std::cout << std::endl;

  float uymin = gPad->GetUymin();
  float uymax = gPad->GetUymax();
  int pymin = gPad->YtoAbsPixel(uymin);
  int pymax = gPad->YtoAbsPixel(uymax);

  //clear last line by xor mode
  if(execstate != kInitial) gVirtualX->DrawLine(pxold,pymin,pxold,pymax);
  gVirtualX->DrawLine(px,pymin,px,pymax);
}

void TArtExec::FGYValue::operator()(EExecState execstate)
{
  if(!gPad) return;
  TObject *select = gPad->GetSelected();
  if(!select) return;
  if(!select->InheritsFrom(TH1::Class())){ return;}

  TCanvas* canvas = gPad->GetCanvas();
  if(!canvas) return;
  if(execstate != kEnd) canvas->FeedbackMode(kTRUE);
  else canvas->FeedbackMode(kFALSE);

  static int py;
  int pyold = py;
  py = gPad->GetEventY();
  std::cout << "\r" << (fY = gPad->AbsPixeltoY(py)) << std::flush;
  if(execstate == kEnd) std::cout << std::endl;

  float uxmin = gPad->GetUxmin();
  float uxmax = gPad->GetUxmax();
  int pxmin = gPad->XtoAbsPixel(uxmin);
  int pxmax = gPad->XtoAbsPixel(uxmax);

  //clear last line by xor mode
  if(execstate != kInitial) gVirtualX->DrawLine(pxmin,pyold,pxmax,pyold);
  gVirtualX->DrawLine(pxmin,py,pxmax,py);
}

void TArtExec::FGXYValue::operator()(EExecState execstate)
{
  if(!gPad) return;
  TObject *select = gPad->GetSelected();
  if(!select) return;
  if(!select->InheritsFrom(TH1::Class())){ return;}

  TCanvas* canvas = gPad->GetCanvas();
  if(!canvas) return;
  if(execstate != kEnd) canvas->FeedbackMode(kTRUE);
  else canvas->FeedbackMode(kFALSE);

  static int px, py;
  int pxold = px;
  int pyold = py;
  px = gPad->GetEventX();
  py = gPad->GetEventY();
  std::cout << "\r"
	    << (fX = gPad->AbsPixeltoX(px)) << " "
	    << (fY = gPad->AbsPixeltoY(py)) << std::flush;
  if(execstate == kEnd) std::cout << std::endl;

  float uxmin = gPad->GetUxmin();
  float uxmax = gPad->GetUxmax();
  int pxmin = gPad->XtoAbsPixel(uxmin);
  int pxmax = gPad->XtoAbsPixel(uxmax);
  float uymin = gPad->GetUymin();
  float uymax = gPad->GetUymax();
  int pymin = gPad->YtoAbsPixel(uymin);
  int pymax = gPad->YtoAbsPixel(uymax);

  if(execstate != kInitial){
    gVirtualX->DrawLine(pxold,pymin,pxold,pymax);
    gVirtualX->DrawLine(pxmin,pyold,pxmax,pyold);
  }
  gVirtualX->DrawLine(px,pymin,px,pymax);
  gVirtualX->DrawLine(pxmin,py,pxmax,py);
}
