
#include "TArtHistManager.hh"
#include "TArtObjLinkManager.hh"
#include "TArtGlobal.hh"

#include "TFile.h"
#include "TDirectory.h"
#include "TList.h"
#include "TKey.h"
#include "TClass.h"
#include "TH1.h"

#include <iostream>
#include <iomanip>
#include <string>

TH1* gTH1 = 0;

TArtObjLinkManager* TArtHistManager::fArtObjLinkManager = new TArtObjLinkManager;

void TArtHistManager::ReadHist(char *filename)
{
  TDirectory* lastdirectory = gDirectory;

  TFile* file = TFile::Open(filename, "READ");
  if(!file || file->IsZombie()) return;

  TList* list = file->GetListOfKeys();
  TObjLink* objlink = list->FirstLink();
  while(objlink){
    TObject* obj = objlink->GetObject();
    if(obj && obj->InheritsFrom(TKey::Class())){
      TKey* key = static_cast<TKey*>(obj);
      obj = key->ReadObj();
      if(obj && obj->InheritsFrom(TH1::Class())){
	TH1* hist = static_cast<TH1*>(obj);
	std::string name = TArtGlobal::IncrementName(hist->GetName(), lastdirectory);
	hist->SetName(name.c_str());
	hist->SetDirectory(lastdirectory);
      }
    }
    objlink = objlink->Next();
  }
  file->Close();

  lastdirectory->cd();
}  

void TArtHistManager::WriteHist(char* filename)
{
  TDirectory* lastdirectory = gDirectory;  

  TFile* file = TFile::Open(filename, "NEW");
  if(!file || file->IsZombie()) return;

  TList* list = lastdirectory->GetList();
  TObjLink* objlink = list->FirstLink();
  while(objlink){
    TObject* obj = objlink->GetObject();
    if(obj && obj->InheritsFrom(TH1::Class())){
      TH1* hist = static_cast<TH1*>(obj);
      file->CloneObject(hist);
    }
    objlink = objlink->Next();
  }
  file->Write();
  file->Close();

  lastdirectory->cd();
}

void TArtHistManager::PrintListOfHist()
{
  bool initial = false;
  if(fArtObjLinkManager->IsInitial()) initial = true;

  TH1* lasthist = GetHist();

  int swidth = 10;
  int mwidth = 30;
  int lwidth = 40;
  std::cout << std::endl;
  std::cout << std::setw(swidth) << std::right << ""
	    << std::setw(swidth) << std::right << "ID"
	    << std::setw(mwidth) << std::right << "Name"
	    << std::setw(lwidth) << std::right << "Title"
	    << std::endl;

  TH1* hist = GetFirstHist();
  TH1* initialhist = hist;
  int count = 1;
  while(hist){
    if(hist == lasthist && !initial){
      std::cout << std::setw(swidth) << std::right << "-->";
    }else{
      std::cout << std::setw(swidth) << std::right << "";
    }
    std::cout << std::setw(swidth) << std::right << count
	      << std::setw(mwidth) << std::right << hist->GetName()
	      << std::setw(lwidth) << std::right << hist->GetTitle()
	      << std::endl;
    hist = GetNextHist();
    if(initialhist == GetHist()) break;
    ++count;
  }
  std::cout << std::endl;

  if(lasthist) SetHist(lasthist);

  if(initial) ResetHist();
}

TH1* TArtHistManager::FindHist(int id)
{
  bool initial = false;
  if(fArtObjLinkManager->IsInitial()) initial = true;

  TH1* lasthist = GetHist();

  TH1* hist = GetFirstHist();
  TH1* initialhist = hist;
  int count = 1;
  while(hist){
    if(count == id) break;
    hist = GetNextHist();
    if(initialhist == GetHist()){
      hist = 0;
      break;
    }
    ++count;
  }

  if(lasthist) SetHist(lasthist);

  if(initial) ResetHist();

  return hist;
}

TH1* TArtHistManager::DrawHist(int id)
{
  TH1* hist = FindHist(id);
  if(hist){
    return DrawHist(hist);
  }else{
    std::cout << " " << id << " do not exist" << std::endl;
  }
}

TH1* TArtHistManager::DrawCurrentHist()
{
  return DrawHist(GetHist());
}

TH1* TArtHistManager::DrawNextHist()
{
  return DrawHist(GetNextHist());
}

TH1* TArtHistManager::DrawPreviousHist()
{
  return DrawHist(GetPreviousHist());
}

TH1* TArtHistManager::DrawHist(TH1* hist)
{
  if(hist){
    hist = SetHist(hist);
    if(hist) hist->DrawCopy(hist->GetOption());
  }    

  return hist;
}

TH1* TArtHistManager::InsertAfterHist(TH1* hist, TH1* histafter)
{
  if(!hist || !histafter) return 0;

  TList* list = fArtObjLinkManager->GetList();
  if(!list) return 0;

  histafter->SetDirectory(0);
  list->AddAfter(hist, histafter);

  return histafter;
}

TH1* TArtHistManager::GetHist()
{
  TObjLink* initialobjlink = fArtObjLinkManager->GetObjLink();
  if(!initialobjlink) return 0;

  TObject* obj;
  while(1){
    obj = (fArtObjLinkManager->GetObjLink())->GetObject();
    if(obj && obj->InheritsFrom(TH1::Class())){
      return gTH1 = static_cast<TH1*>(obj);
    }else{
      fArtObjLinkManager->NextObjLink();
      if(initialobjlink == (fArtObjLinkManager->GetObjLink())) break;
      continue;
    }
  }

  return 0;
}

TH1* TArtHistManager::SetHist(TH1* hist)
{
  if(fArtObjLinkManager->SetObjLink(hist)){
    return gTH1 = hist;
  }else{
    return 0;
  }
}

void TArtHistManager::ResetHist()
{
  gTH1 = 0;
  fArtObjLinkManager->ResetObjLink();
}

TH1* TArtHistManager::GetNextHist()
{
  TObjLink* initialobjlink = fArtObjLinkManager->NextObjLink();
  if(!initialobjlink) return 0;

  TObject* obj;
  while(1){
    obj = (fArtObjLinkManager->GetObjLink())->GetObject();
    if(obj && obj->InheritsFrom(TH1::Class())){
      return gTH1 = static_cast<TH1*>(obj);
    }else{
      fArtObjLinkManager->NextObjLink();
      if(initialobjlink == (fArtObjLinkManager->GetObjLink())) break;
      continue;
    }
  }

  return 0;
}

TH1* TArtHistManager::GetPreviousHist()
{
  TObjLink* initialobjlink = fArtObjLinkManager->PreviousObjLink();
  if(!initialobjlink) return 0;

  TObject* obj;
  while(1){
    obj = (fArtObjLinkManager->GetObjLink())->GetObject();
    if(obj && obj->InheritsFrom(TH1::Class())){
      return gTH1 = static_cast<TH1*>(obj);
    }else{
      fArtObjLinkManager->PreviousObjLink();
      if(initialobjlink == (fArtObjLinkManager->GetObjLink())) break;
      continue;
    }
  }

  return 0;
}

TH1* TArtHistManager::GetFirstHist()
{
  TObjLink* objlink = fArtObjLinkManager->FirstObjLink();

  while(objlink){
    TObject* obj = objlink->GetObject();
    if(obj && obj->InheritsFrom(TH1::Class())){
      return gTH1 = static_cast<TH1*>(obj);
    }else{
      objlink = objlink->Next();
      continue;
    }
  }

  return 0;
}

TH1* TArtHistManager::GetLastHist()
{
  TObjLink* objlink = fArtObjLinkManager->LastObjLink();

  while(objlink){
    TObject* obj = (fArtObjLinkManager->GetObjLink())->GetObject();
    if(obj && obj->InheritsFrom(TH1::Class())){
      return gTH1 = static_cast<TH1*>(obj);
    }else{
      objlink = objlink->Prev();
      continue;
    }
  }

  return 0;
}
