#include "TArtCalibNEBULA.h" 
#include "TArtNEBULA.h"
#include "TArtNEBULAPara.h"
#include "TArtNEBULAParameters.h"
#include "TArtRawEventObject.h"
#include "TArtCore.h" 
#include "TArtMapper.h"

#include <TROOT.h>

//__________________________________________________________
TArtCalibNEBULA::TArtCalibNEBULA()
  :fReconstructed(false)
{
  TArtCore::Info(__FILE__,"Creating the NEBULA detector objects...");
  setup = (TArtNEBULAParameters*)gROOT->FindObject("NEBULAParameters");
  fNEBULA = new TArtNEBULAArray();
  fNEBULA->SetOwner();
}

//__________________________________________________________
TArtCalibNEBULA::TArtCalibNEBULA(TArtNEBULAParameters * s)
  :fReconstructed(false), setup(s)
{
  TArtCore::Info(__FILE__,"Creating the NEBULA detector objects...");
  fNEBULA = new TArtNEBULAArray();
  fNEBULA->SetOwner();
}

//__________________________________________________________
TArtCalibNEBULA::~TArtCalibNEBULA()  {
  fNEBULA->Delete();
}

//__________________________________________________________
void TArtCalibNEBULA::LoadRawData(TArtRawEventObject *fEvent)   {

  ClearData();
  for(int i=0;i<fEvent->GetNumSeg();i++){
    TArtRawSegmentObject *seg = fEvent->GetSegment(i);
    int device   = seg->GetDevice();
    int fpl      = seg->GetFP();
    int detector = seg->GetDetector();
    if(SAMURAI == device)//will be NEBULA??
      for(int j=0;j<seg->GetNumData();j++){
	TArtRawDataObject *d = seg->GetData(j);
	int geo = d->GetGeo(); 
	int ch = d->GetCh(); 
	unsigned int val = d->GetVal();
	TArtRIDFMap mm(fpl,detector,geo,ch);
	TArtNEBULAPara *para = (TArtNEBULAPara*)setup->GetNEBULAPara(&mm);
	if(NULL == para){
	  TArtCore::Info(__FILE__,"Could not find TArtNEBULAPara...: Dev:%d, %s", device, mm.GetMapInfo());
	  continue;
	}
	if(para->GetIgnore()){
	  TArtCore::Debug(__FILE__,"Ignore TArtNEBULAPara...: Dev:%d, %s", device, mm.GetMapInfo());
	  continue;
	}
	int id = para->GetID();
	int npla = fNEBULA->GetEntries();
	TArtCore::Debug(__FILE__,"Entries in NEBULAArray: %d, NEBULA-ID:%d", npla, id);
	TArtNEBULA * pla = fNEBULA->FindPla(id);

	if(pla==NULL){
	  new ((*fNEBULA)[npla]) TArtNEBULA(para);
	  pla = (TArtNEBULA *)fNEBULA->At(npla);
	}

	// set raw data
	if(42 == detector)
	  //	if(NEBULAA == detector)
	  pla->SetRawADC(val);
	if(41 == detector)
	  //	if(NEBULAT == detector)
	  pla->SetRawTDC(val);

      }
  }
  return;

}

//__________________________________________________________
void TArtCalibNEBULA::ClearData()   {

  (*fNEBULA).Delete();
  fReconstructed = false;
  return;

}

void TArtCalibNEBULA::ReconstructData(TArtRawEventObject *fEvent)   {

  LoadRawData(fEvent);
  ReconstructData();

  return;
}

//__________________________________________________________
void TArtCalibNEBULA::ReconstructData()   { // call after the raw data are loaded

  fNEBULA->ReconstructData();
  fReconstructed = true;
  return;

}

//__________________________________________________________
TArtNEBULA * TArtCalibNEBULA::GetNEBULA(int i) {
  return (TArtNEBULA *)fNEBULA->At(i);
}
//__________________________________________________________
Int_t TArtCalibNEBULA::GetNumNEBULA() {
  return fNEBULA->GetEntries();
}
